/*************************************************************************

	File Name: vcr.c

	Description: This file contains the VBX implementation for the
	Super Classed VCR Button.  This is compiled as a VBX file which
	is really a DLL.  This bit of code allows the VCR button to be 
	used by both Visual Basic 2.0 and Visual C++ App Studio.

	Functions: 
		LibMain
		VBGetModelInfo
		VBINITCC
		VcrCtlProc
**************************************************************************/

#include <windows.h>
#include <vbapi.h>  // Provided as part of the VBX tool kit from Microsoft
#include "vcr.h"

/*
** Global instance of the VBX module.  This is set in LibMain.
*/

HANDLE hInst;

/*
** Declare a Custom Property for the Bevel width
** of the VCR Button.  The VCR Button allows the bevel 
** width to be set externally.  By declaring a custom
** property the Bevel Width property appears in the
** property window. 
**
** The parameters here declare the Bevel Width as:
**	A Short - Minimum Width of 2
**	A Property Message is sent to the control to set the
**     bevel width property.
**     The set value is saved as part of the control definition
**	
** The offset macro calculates the offset into the user defined
** data area associated with the control.
*/

PROPINFO Property_Bevel =
{
"BevelWdith",
DT_SHORT | PF_fSetMsg | PF_fSaveData | PF_fGetData | PF_fDefVal,
OFFSETIN( PUSH, bevel_width ),
0,2,NULL,0 				     
};

/*
** The Value Property allows the program to set the state
** of the control.  This is a BOOL and is passed to the 
** underlying control.
*/

PROPINFO Property_Value =
{
"Value",
DT_BOOL | PF_fSetMsg | PF_fSaveData | PF_fGetData | PF_fDefVal,
OFFSETIN( PUSH, value ),
0,0,NULL,0 				     
};

/*
** Property list for the VCR button.  This list defines the
** standard properties that are available to the VCR Button.
**
** The two custom properties are listed at the end. 
*/

PPROPINFO proplist[] =
{
    PPROPINFO_STD_NAME,
    PPROPINFO_STD_INDEX,
    PPROPINFO_STD_PARENT,
    PPROPINFO_STD_LEFT,
    PPROPINFO_STD_TOP,
    PPROPINFO_STD_WIDTH,
    PPROPINFO_STD_HEIGHT,
    PPROPINFO_STD_ENABLED,
    PPROPINFO_STD_VISIBLE,
    PPROPINFO_STD_CAPTION,
    PPROPINFO_STD_FONTNAME,
    PPROPINFO_STD_FONTSIZE,
    PPROPINFO_STD_FONTBOLD,
    PPROPINFO_STD_FONTITALIC,
    PPROPINFO_STD_FONTSTRIKE,
    PPROPINFO_STD_FONTUNDER,
    PPROPINFO_STD_TABINDEX,
    PPROPINFO_STD_TABSTOP,
    PPROPINFO_STD_TAG,
    PPROPINFO_STD_HWND,
    &Property_Bevel,
    &Property_Value,
    NULL
};

/*
** Since the under lying code grabs the mouse when a mouse
** down is detected, it is necessary to create a custom button 
** event.  This does nothing but fire an event that the underlying
** code uses.  Since we have declared Value as a property it becomes
** global in Visual Basic 2.0 and can be set at any time.
*/

EVENTINFO Event_Click =
    {
    "Click",
    0,
    0,
    NULL,
    NULL 
    };

/*
** List of supported Events for the control
** The first event is the custom click event
** The rest are standard events that insure that
** the correct set of messages are sent through
** to the underlying Super Class Button.
*/

PEVENTINFO eventlist[] =
{
    &Event_Click,
    PEVENTINFO_STD_GOTFOCUS,
    PEVENTINFO_STD_LOSTFOCUS,
    PEVENTINFO_STD_KEYPRESS,
    PEVENTINFO_STD_KEYDOWN,
    PEVENTINFO_STD_KEYUP,
    NULL
};

/*
** Model structure for Version 1.00 VB Controls
** Define version 1.00 as a start.  This will be patched
** when the request call is made.
**
** The important features here are the use of the 
** MODEL_fMnemonic flag to get key translation as
** button clicks and the use of the "VCRButton"
** class to inherit the underlying behavior 
** of the VCR Button.
**
** A border style is also defined with a WS_BORDER.
** This is not a default for Visual Basic Controls.
** Without the border the 3D effect can be lost
** on some backgrounds.
*/

MODEL model_VCR =
{
    VB100_VERSION,	  			// VB version being used
    MODEL_fFocusOk | MODEL_fMnemonic,	// MODEL flags
    (PCTLPROC)VcrCtlProc,			// Control procedure
    CS_VREDRAW | CS_HREDRAW,		// Class style
    WS_VISIBLE | WS_BORDER,			// Default Window style
    sizeof(PUSH),				// cbCtlExtra for PUSH structure
    IDBMP_PUSH, 				// Palette bitmap ID
    "Vcr",					// Default control name
    "VbxButton",				// Visual Basic class name
    "VCRButton",				// Parent class name
    proplist,					// Properties list
    eventlist,				// Events list
    IPROP_CAPTION, 				// Default property
    EVENT_CLICK,				// Default event
    IPROP_CAPTION				// Property representing value of ctl
};

/*
** Define an array of models for this control.
** Here we only point to one control in this module
** If the module contained more than one control
** they would be part of this array.
*/

LPMODEL model_list[] =
{
    &model_VCR,
    NULL
};

MODELINFO model_info =
{
    VB100_VERSION,			
    model_list	      
};

/*********************************************************************

Function Name: LibMain

Purpose: Initialize library. This routine is called when the 
     	   first client loads the DLL

**********************************************************************/
int FAR PASCAL LibMain( HANDLE hModule, WORD wDataSeg, WORD cbHeapSize, LPSTR lpszCmdLine )
{
	// Avoid warnings on unused (but required) formal parameters

	wDataSeg = wDataSeg;
	cbHeapSize = cbHeapSize;
	lpszCmdLine = lpszCmdLine;

	hInst = hModule;

	return 1;
}
/*********************************************************************

Function Name: VcrCtlProc

Purpose: Control procedure for the VBX control.  Messages for a VBX
	  control are sent here first for processing.  Those messages
	  that map to window messages are later forwarded to the 
	  normmal VCR window procedure for processing.

**********************************************************************/
LONG FAR PASCAL _export VcrCtlProc( HCTL hCtl, HWND hWnd, USHORT msg, 
	  USHORT wp, LONG lp )
{
	LONG style;
	WORD bevel;
	BOOL value;
	PUSH FAR * lpPush;
	static char buf[40];
	
	/*
	** Do all message preprocessing here.  Since the VBX button
	** does not use the custom windows style we need to set it 
	** here when the VCR button is first created.  Remember the
	** underlying VCR button can also emulate the radio button.
	** This is changed here based on a VBX property.
	*/

	switch (msg)
	{
	
	/*
	** Trap out all key translations and pass them to
	** the control as clicks.
	*/
    
	case VBM_MNEMONIC:

	    lp = MAKELONG(0,BN_CLICKED);
	  
	    // **** FALL THROUGH ****

	case VBN_COMMAND:

		/*
		** Handle anything that looks like a button click
		** This is the handling for the custom event that
		** was defined above.  Here we simply take a BN_CLICK
		** and pass it along the the VBX by using the 
		** fire event.
		*/

		switch (HIWORD(lp))
		{
		case BN_CLICKED:

		    VBFireEvent( hCtl, EVENT_CLICK, NULL );
		    break;
		    
		}
	    	break;

	/*
	** Whenever a property is changed a message is sent here.  This
	** ocurrs in both design and run mode.  There is no need to
	** handle the standard properties that are declared above
	** as they are handled automatically.  Custom properties
	** do need to be processed.  Since both properties are 
	** supported by the underlying super classed control, the
	** values are simply passed to the underlying code.  
	**
	** Here we handle only Set Property. This can cause the
	** uper layer to become out of sync with the upper layer
	** at startup.  A Get Property could also be added.
	*/

    	case VBM_SETPROPERTY:
		
		switch( wp )
		{
			case IPROP_BEVEL:
			
				bevel = (WORD) lp; 
			
				SendMessage( hWnd, BM_SETBORDER, bevel, 0L );
				if ( lpPush = (PUSH FAR *) VBDerefControl( hCtl ))
				{
 					lpPush->bevel_width = bevel;
					return( 0L );
				}				
				break;
	
			case IPROP_VALUE:
			
				value = (BOOL) lp; 
			
				SendMessage( hWnd, BM_SETCHECK, (value) ? TRUE : FALSE, 0L );
				if ( lpPush = (PUSH FAR *) VBDerefControl( hCtl ))
				{
 					lpPush->value = value;
					return( 0L );
				}				
				break;
			}
		break;

    	/*
	** Since the model definition does not support passing
	** user defined Style Flags to the underlying super classed control
	** thay are added here.  This causes the base control to 
	** act like to the VCR Button.
	*/
    
       case WM_NCCREATE:
         
       	style = GetWindowLong( hWnd, GWL_STYLE );
            	style |= BSS_VCRBUTTON;
            	SetWindowLong( hWnd, GWL_STYLE, style );
            	break;
            
	default:
            	break;
	}

	/*
	** The default processing is to call the VBX control
	** procedure.  
	*/

	 return( VBDefControlProc( hCtl, hWnd, msg, wp, lp) );
}

/*********************************************************************

Function Name: VBGetModelInfo

Purpose: This function is required by the VBX to get property 
	  information about the control.  based on the version
	  return the address of an structure that contains the
	  supported properties for the control.

**********************************************************************/
LPMODELINFO FAR PASCAL _export VBGetModelInfo( USHORT usVersion )
{	
	/*
	** Set up model version.  Since we support
	** both versions with identical parameters
	** we need only patch in the passed version.
	** 
	** If the special features of VB Version 2.0
	** were to be supported then two different 
	** info sturctures would be needed.
	*/	   	
	model_info.usVersion = usVersion;
	return( &model_info );
}

/*********************************************************************

Function Name: VBINITCC

Purpose: Register custom control. This routine is called by VBX 
	  when the custom control DLL is first loaded.  This is 
	  different than the Windows Register Class.  All controls
	  used as VBX controls must be registered this way.

**********************************************************************/
BOOL FAR PASCAL _export VBINITCC( USHORT usVersion, BOOL	fRuntime )
{
	// Avoid warnings on unused (but required) formal parameters

	fRuntime = fRuntime;

	/*
	** Register control properties for the VBX
	** control.  These properites control the 
	** design and operation of the control
	*/

	model_VCR.usVersion = usVersion;
	return( VBRegisterModel( hInst, &model_VCR ) );
}
