/* ----------------------------------------------------------------
 *	FILE vcrbut.c
 *
 *	DESCRIPTION: This file contains code to implement a VCR
 *		button as a super class of the standard radio
 *		button.  The support provided by this file
 *		is 'C' code and provides a global class as a DLL.
 *
 *	INTERFACE ROUTINES
 *		CustomButtonInit
 *		BtnWalk
 *		FocusBorder
 *		chk_btn_style
 *		WndProcButton
 *
 *	IDENTIFICATION
 *		Code provided as a courtesy of Scopus Technology Inc. 
 *		Emeryville, CA.
 *		Code Revision and Preparation for Publication done by:
 *		Bill Breedlove, Evergreen Productions.
 *
 * ----------------------------------------------------------------*/

#define STRICT
#define _WINDLL

#define NOMINMAX
#define NOCOMM
#define NOICONS
#define NOKEYSTATES
#define NOSYSCOMMANDS
#define NOOEMRESOURCE
#define NOATOM
#define NOCLIPBOARD
#define NOSOUND
#define NOWH
#define NOKANJI
#define NOHELP
#define NOPROFILER
#define NODEFERWINDOWPOS

#include <windows.h>  
#include <string.h>

#include "vcrbut.h"		// Public definations
#include "winctl.h"		// Local defintions

/*===========================================================================
                          Global Variable Area
===========================================================================*/

HANDLE hInst;			// current instance of DLL
WNDPROC OldButtonProc;	// Address to under lying button procedure
int extra;			// Offset to custom defined window extra bytes

/*DOC*************************************************************************
Function Name: CustomButtonInit( hInst )

Purpose: This function registers the class for the VCR button control
	It is called from LibMain.  This allows custom windows classes
	to be registered when the module is loaded, insuring that 
	the class is always available to the application.

Entry: Handle to DLL instance

Returns: Result of the RegisterClass()

Comments: LibMain will fail if the class is not registered.  This
	means that the application will also fail to load.  
	Unfortunately, it is not possible to bring up a message
	box while the DLL is being loaded.  Generally the class
	registration will not fail.  If error tracking is a
	requirement then error messages can be written to a 
	file at the load time.

**DOC************************************************************************/
BOOL VCRButtonInit( HINSTANCE hInst )
{
	WNDCLASS WndClass;

	/*
	** Get the under lying class to use.  Since the radio
	** button is a style of button the class information 
	** is requested from windows.
	*/

	GetClassInfo( NULL, "button", &WndClass );

	/*
	** Save the address of the button window
	** procedure.  This will be used by the super 
	** classed VCR button to pass the messages 
	** that need no additionaly processing.
	*/

	OldButtonProc = WndClass.lpfnWndProc;

	/*
	** Register the new class
	** The class is made global so that
	** all applications can create VCR buttons
	*/

	WndClass.style		|= CS_GLOBALCLASS;
	WndClass.lpfnWndProc		= (WNDPROC) WndProcButton;
	extra = WndClass.cbWndExtra;
	WndClass.cbWndExtra		+= BTN_EXTRA;
	WndClass.hInstance		= hInst;
	WndClass.hbrBackground	= (HBRUSH) GetStockObject(BLACK_BRUSH);
	WndClass.lpszMenuName	= NULL;
	WndClass.lpszClassName	= (LPSTR) "VCRButton";

	return( RegisterClass( &WndClass ) );
}

/*****************************************************************************
Function: three_D_border

Purpose: This function highlights a control and creates a three D effect.
	This is don within the client area of the control.  This could be
	modified to use the non client area if alot of drawing was done on
	the client area.

Entry: Handle to window to highlight
	mode - pushed in or popped out
		 TRUE - popped out
		 FALSE - pushed in

*****************************************************************************/
void three_D_border( HWND hWnd, int mode )
{
	RECT rect;
	HPEN hPen, hPenOld;
	HBRUSH hBrush, hBrushOld;
	COLORREF tmp_clr;
	int border_width;
	POINT pt[8];
	
	HDC hDC = GetDC( hWnd );
	GetClientRect( hWnd, &rect );

	/*
	** Get the internal border width for
	** this instance of the control.
	*/
	
	border_width = GetWindowWord( hWnd, extra + BTN_BORDER ) & 0x00ff;
	
	if ( mode & 0x0001 )
		tmp_clr = GetSysColor( COLOR_BTNFACE );
	else
		tmp_clr = GetSysColor( COLOR_BTNHIGHLIGHT );

	hPen = CreatePen( PS_SOLID, 1, tmp_clr );
	hBrush = CreateSolidBrush( tmp_clr );
	hPenOld = SelectObject( hDC, hPen );
	hBrushOld = SelectObject( hDC, hBrush );

	/*
	** Draw the top border
	** Here we use the polygon so that we get
	** diagonal corners.  This is not important
	** on a thin border but makes a difference
	** when the border is wider than 3 pixels.
	*/

	pt[0].x = pt[6].x = rect.left;
	pt[0].y = pt[6].y = rect.top;
	pt[1].x = border_width - 1;
	pt[1].y = pt[2].y = border_width - 1;
	pt[2].x = pt[3].x = rect.right - border_width;
	pt[3].y = rect.bottom - border_width;
	pt[4].x = pt[5].x = rect.right - 1;
	pt[4].y = rect.bottom - 1;
	pt[5].y = rect.top;
	
	Polygon( hDC, pt, 7 );

	SelectObject( hDC, hPenOld );
	DeleteObject( hPen );
	SelectObject( hDC, hBrushOld );
	DeleteObject( hBrush );

	if ( mode & 0x0001 )
		tmp_clr = GetSysColor( COLOR_BTNHIGHLIGHT );
	else
		tmp_clr = GetSysColor( COLOR_BTNSHADOW );

	hPen = CreatePen( PS_SOLID, 1, tmp_clr );
	hBrush = CreateSolidBrush( tmp_clr );
	hPenOld = SelectObject( hDC, hPen );
	hBrushOld = SelectObject( hDC, hBrush );

	/*
	** Draw the under line
	*/
	pt[0].x = pt[1].x = pt[6].x = rect.left;
	pt[0].y = pt[6].y = rect.top;
	pt[1].y = pt[2].y = rect.bottom - 1;
	pt[2].x = rect.right - 2;
	pt[3].x = rect.right - border_width - 1;
	pt[3].y = rect.bottom - border_width;
	pt[4].x = pt[5].x = border_width - 1;
	pt[4].y = rect.bottom - border_width;
	pt[5].y = rect.top + border_width - 1;
	
	Polygon( hDC, pt, 7 );

	/*
	** clean up before exit
	*/

	SelectObject( hDC, hPenOld );
	DeleteObject( hPen );
	SelectObject( hDC, hBrushOld );
	DeleteObject( hBrush );
	
	if ( mode & 0x0004 )
	{
		hPen = CreatePen( PS_SOLID, border_width / 3 + 1, 0 );
		hPenOld = SelectObject( hDC, hPen );
		hBrushOld = SelectObject( hDC, GetStockObject( NULL_BRUSH ));
		Rectangle( hDC, rect.left, rect.top, rect.right, rect.bottom );
		SelectObject( hDC, hPenOld );
		SelectObject( hDC, hBrushOld );
		DeleteObject( hPen );
	}

	ReleaseDC( hWnd, hDC );
	return;
}

/*DOC********************************************************************
Function Name: button_walk

Purpose: This function is used by the auto radio button style.  All
	VCR buttons in a group are checked and the state set to
	off.  This allows automatic checking of VCR buttons.

Entry: Starting window handle
	Direction to walk the window list.

**DOC********************************************************************/
static void button_walk( HWND hWnd, WORD wDirection )
{
HWND hTmp;
static char class_name[ 33 ];
LONG lStyle;
int tmp_chk;

	hTmp = hWnd;

	while ( hTmp = GetWindow( hTmp, wDirection ) )
	{
		/*
		** Get the class name and check for
		** the button class of some sort.  This
		** is a bit dangerous as some class names may
		** contain button and not conform to the 
		** style bits.  To limit this to VCRBUTTON
		** then use a strcmp().
		*/
	
		GetClassName( hTmp, class_name, 32 );
		_fstrupr( class_name );
		if ( _fstrstr( class_name, "BUTTON" ) )
			return;
	
		/*
		** Get the style and look for the AutoRadio
		** style bit.  The BSS_AUTOVCRBUTTON flag
		** automatically sets this bit.
		*/
			
		lStyle = GetWindowLong( hTmp, GWL_STYLE );
		if ( lStyle & ( BS_AUTORADIOBUTTON ) )
		{
			/*
			** stop if we reach next group
			*/

			if ( lStyle & WS_GROUP )
				break;

			/*
			** A valid radio button found, make sure it is
			** unchecked.  Do this internally so that we
			** do not generate additional auto walk.
			*/

			if (( tmp_chk = GetWindowWord( hTmp, extra )) & 0x0001 )
			{
				three_D_border( hTmp, tmp_chk );
				SetWindowWord( hTmp, extra, tmp_chk &= ~0x0001 );
			}
		}
		else
			break;
	}
}

/*DOC********************************************************************
Function Name: focus_border

Purpose: This function displays a focus border for the VCR
	button.  The focus border is a dashed rectangle around
	the text.

Entry:	Handle to a control
	Mode - add or remove focus border

**DOC********************************************************************/
static void focus_border( HWND hWnd, int Mode )
{
RECT rect;
HDC hDC;
int border_width;

	/*
	** Get the current border width
	*/
	
	border_width = GetWindowWord( hWnd, extra + BTN_BORDER ) & 0x00ff;
	
	/*
	** Always draw this on the client rectangle
	*/

	hDC = GetDC( hWnd );
	GetClientRect( hWnd, &rect );
	InflateRect( &rect,
	 (GetSystemMetrics( SM_CXBORDER ) * -2) - border_width, 
	 (GetSystemMetrics( SM_CYBORDER ) * -2) - border_width );

	/*
	** Put up or take down the focus rectangle
	*/

	DrawFocusRect( hDC, &rect );

	ReleaseDC( hWnd, hDC );
}

/*DOC********************************************************************
Function Name: chk_btn_style

Purpose: This is a helper function that helps separate style bits
	and returns the result.

Entry:	Handle to a control

**DOC********************************************************************/
static int chk_btn_style( HWND hWnd )
{
LONG style;

	style = GetWindowLong( hWnd, GWL_STYLE );

	 if ( (style & ( BS_AUTORADIOBUTTON )) == BS_AUTORADIOBUTTON )
		return( BS_AUTORADIOBUTTON );

	 if ( (style & ( BS_RADIOBUTTON )) == BS_RADIOBUTTON )
		return( BS_RADIOBUTTON );

	return( 0 );
}

/*DOC********************************************************************
Function Name: WndProcVCRButton

Purpose: This is the main window procedure for the VCR button.  Any
	messages that need special handling are processed here.  The
	rest of the messages to support regular button processing
	are passed to the regular button window procedure via the
	address saved when the VCR button class is registered.

Entry:	Standard Windows Parameters.

**DOC********************************************************************/
LRESULT CALLBACK 
WndProcButton( HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
LONG lStyle;
RECT rect;
HFONT hFont;
WORD tmp_chk;

	switch ( wMsg )
	{

	/*
	** Do some extra processing if we are in the
	** custom mode.  In this case we set a default border
	** width for the VCR button.
	*/
	
	case WM_CREATE:
	
		if ( GetWindowLong( hWnd, GWL_STYLE ) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) || 
			 (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
				SendMessage( hWnd, BM_SETBORDER, DEFAULT_BTN_BORDER, 0L );

		break;

	/*
	** Handle the calculation of the non client area.  The
	** normal radio button reserves no area for a border
	** Here we pass the WM_NCCALCSIZE to the default
	** window procedure which will calc an area based on
	** the type of border the VCR Button has.
=	*/

	case WM_NCCALCSIZE:

		if (( lStyle = GetWindowLong( hWnd, GWL_STYLE )) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) || 
			 (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
				return( DefWindowProc( hWnd, wMsg, wParam, lParam ));
		
		break;

	/*
	** WM_USER + 1
	** Set or clear a check from the radio button.  In the
	** case of the VCR code we invert - push in or pull out
	** the button.  This messages must be handled since the 
	** underlying windows control code paints the state of
	** the button on this message.
	*/

	case BM_SETCHECK:

		if ( GetWindowLong( hWnd, GWL_STYLE ) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) || 
			 (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
			{
	
				/*
				** If the auto style is set for the VCR button
				** then walk the list of other VCR buttons.
				** This is done for the buttons both before 
				** and after the current button. 
				*/
	
				if ( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON )
				{
					button_walk( hWnd, GW_HWNDPREV );
					button_walk( hWnd, GW_HWNDNEXT );
				}
	
				/*
				** Get the previous set state from the first
				** byte of the first window word.  The on-off
				** state is stored in the least significant
				** bit.  We toggle this bit but preserve
				** the rest of the bits.
				*/
		
				tmp_chk = GetWindowWord( hWnd, 0 );
				if ( wParam )
					tmp_chk |= 0x0001;
				else
					tmp_chk &= ~0x0001;
				SetWindowWord( hWnd, 0, tmp_chk );
		
				/*
				** Since we are generally running on a fast
				** system and the toggling of the visual state
				** involves the entire button, it is easier
				** to issue a re-paint for the entire button.
				*/
				
				InvalidateRect( hWnd, NULL, FALSE );
				
				/*
				** Block any further processing 
				** this must be done to prevent the old
				** radio button from being drawn,
				*/
				
				return( 0L );
			}
			
		/*
		** If we get here then this button style does not 
		** require special processing and the default
		** routines are called.  This allows the VCR 
		** button class to support all other button
		** styles.
		*/

		break;

	/*
	** WM_USER + 3
	** This message is sent to the control when it is
	** first selected or de-selected.  The default action 
	** is to highlight the control button.  Again this
	** message must be handled since it does direct screen
	** drawing.
	*/
	
	case BM_SETSTATE:

		/*
		** Check for special button processing
		*/

		if ( GetWindowLong( hWnd, GWL_STYLE ) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) 
			 || (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
			{
			
				/*
				** Get the current selection state
				** without destroying the other
				** state bits.
				*/
			
				tmp_chk = GetWindowWord( hWnd, 0 );
				
				/*
				** Put in or take out the selected
				** state bit based on the passed 
				** state.  The state bit is documented
				** in the Windows SDK.
				*/
	
				if ( wParam )
					tmp_chk |= 0x0004;
				else
					tmp_chk &= ~0x0004;
				SetWindowWord( hWnd, 0, tmp_chk );
				three_D_border( hWnd, tmp_chk );
	
				/*
				** Add button highlight call here
				** to draw a black border around the control
				*/
		
				/*
				** Block default processing
				*/
				
				return( 0L );
			}
	
		break;

	/*
	** Set the border width for the control.  In the visual
	** products this is handled as a property.  Since the 
	** underlying code must respond to a different size 
	** border, we keep it here as a an extra byte.  This
	** opens this capability up to a normal 'C' program
	*/

	case BM_SETBORDER:
	
		tmp_chk = GetWindowWord( hWnd, extra + BTN_BORDER );
		
		/*
		** Put in the size of the border in the 
		** low byte.  The high byyte is reserved
		** for some style flags.
		**
		** Make sure that the border width is at least
		** 2 wide and less than 64.
		*/
	
		tmp_chk &= 0xFF00;
		if ( wParam < 2 )
			wParam = 2;
		tmp_chk |= wParam & 0x0003F;
		SetWindowWord( hWnd, extra + BTN_BORDER, tmp_chk );
		return( 0L );

	/*
	** Since we are handling the painting of
	** the VCR button we also have to handle the
	** situation where the control is disabled.
	** Windows will handle all other stuff related
	** to the control being disabled.
	*/
	
	case WM_ENABLE:

		if ( GetWindowLong( hWnd, GWL_STYLE ) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) 
			 || (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
			{
				
				/*
				** Just let the paint routine
				** display the disabled state
				*/
				
				InvalidateRect( hWnd, NULL, FALSE );
				return( 0L );
			}
		break;

	/*
	** On a control we also draw a focus rectangle when
	** the control has focus.  This is typically done
	** drawing a dashed rectangle around the button text.
	** In the case of the radio button the set and kill
	** focus message draws and removes this rectangle.
	*/

	case WM_SETFOCUS:
	case WM_KILLFOCUS:

		if ( GetWindowLong( hWnd, GWL_STYLE ) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) 
			 || (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
			{
			WORD tmp_chk;
	
				/*
				** Toggle the focus bit
				*/
			
				tmp_chk = GetWindowWord( hWnd, 0 );
	
				if ( wMsg == WM_SETFOCUS )
					tmp_chk |= 0x0008;
				else
					tmp_chk &= ~0x0008;
		
				SetWindowWord( hWnd, 0, tmp_chk );
	
				/*
				** Draw the focus rectangle
				*/
				
				focus_border( hWnd, ( GetFocus() == hWnd ) );
	
				/*
				** Block drawing the normal radio button
				** focus rectangle.
				*/
				
				return( 0L );
			}
		break;
	

	/*
	** Handle any non-client Painting.  This is generally the
	** border.  Here we use the default windows processing.
	*/	

	case WM_NCPAINT:

		/*
		** Do radio button processing
		*/

		if ( GetWindowLong( hWnd, GWL_STYLE ) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) 
			 || (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
			{
				
				/*
				** Nothing to do yet
				*/
				
				break;
			}

		break;

	/*
	** This is what make the VCR button special.  All the
	** display level is handled here.
	*/
	
	case WM_PAINT:
	{
	char szText[ MAX_TEXT_SIZE + 1 ];
	TEXTMETRIC tm;
	int iCenter;
	int iTop;
	PAINTSTRUCT ps;
	HBRUSH hBrush;
	COLORREF tmp_clr;

		/*
		** Check for VCR radio button special processing
		*/

		if ( GetWindowLong( hWnd, GWL_STYLE ) & BSS_VCR )
			if (( chk_btn_style( hWnd ) == BS_AUTORADIOBUTTON ) 
			 || (chk_btn_style( hWnd ) == BS_RADIOBUTTON ))
			{
				
				BeginPaint( hWnd, &ps );

				/*
				** Find out window size to draw the underline
				*/
				
				GetClientRect( hWnd, &rect );

				/*
				** Draw button text in button centered, then 
				** a rectangle border
				*/

				SetTextAlign( ps.hdc, TA_CENTER );

				/*
				** Get any control assigned font to use
				** from the underlying font table in the control
				** and select it for drawing.  We could just
				** get a font handle from the extra words.  This
				** would be quicker.  Sending a message is more
				** portable to future versions of Windows.
				*/

				if ( hFont = (HFONT) LOWORD( SendMessage( hWnd,
				 WM_GETFONT, 0, 0L )) )
					hFont = SelectObject( ps.hdc, hFont );
	
				/*
				** Here we take the Microsoft easy way out.  Generally
				** we would send a WM_CTLCOLOR message here to
				** allow the parent to set the color of the button.
				** Instead we follow Microsoft's lead and use the
				** system defined button colors.  These can be changed
				** for all buttons in the WIN.INI file.  This 
				** allows the VCR button to match all other push
				** buttons in the system.  A nice enhancement
				** here would be to merge the brush color with
				** the Windows System defined colors to create
				** some special effect buttons.  Another nice
				** enhancement that could be added at this point is
				** to paint the button with a bit map.
				**
				** To make the pushed in or selected state of
				** the button standout we use the BTNSHADOW
				** as the background instead of the BTNFACE.
				*/

				if ( GetWindowWord( hWnd, 0 ) & 0x0001 )
				{
					tmp_clr = GetSysColor( COLOR_BTNSHADOW );
					SetBkColor( ps.hdc, tmp_clr  );
					SetTextColor( ps.hdc, GetSysColor( COLOR_BTNHIGHLIGHT ) );
				}
				else
				{
					tmp_clr = GetSysColor( COLOR_BTNFACE );
					SetBkColor( ps.hdc, tmp_clr );
					SetTextColor( ps.hdc, GetSysColor( COLOR_BTNTEXT ) );
				}

				/*
				** Fill the button with the current solid color.
				*/

				hBrush = CreateSolidBrush( tmp_clr );
				FillRect( ps.hdc, &rect, hBrush );
				DeleteObject( hBrush );

				/*
				** Now handle the text for the button.  Here
				** we impose a limit on the text that can
				** be put in a push button.  This can be
				** increased by modfing the local header
				** for the control.
				**
				** The text is centered by default.  This could
				** be modified by a property to allow left and
				** right text.
				*/

				GetWindowText( hWnd, szText, MAX_TEXT_SIZE );
				GetTextMetrics( ps.hdc, &tm );
				iCenter = ( rect.right - rect.left ) / 2;
				iTop = (rect.bottom - tm.tmHeight) / 2;

				/*
				** Check to see if we need to gray the string
				*/
				
				if ( IsWindowEnabled( hWnd ) == FALSE )
				{
				if ( tmp_clr == GetSysColor( COLOR_BTNSHADOW ) )
					tmp_clr = GetSysColor( COLOR_GRAYTEXT );
				else
					tmp_clr = GetSysColor( COLOR_BTNSHADOW );

				SetTextColor( ps.hdc, tmp_clr );
				}
	
				TextOut( ps.hdc, iCenter, iTop, szText, lstrlen( szText ) );

				/*
				** If we are using a special font then deselect it
				*/

				if ( hFont )
					hFont = SelectObject( ps.hdc, hFont );

				/*
				** Redraw the the focus rectangle if
				** we have focus.
				*/

				three_D_border( hWnd, GetWindowWord( hWnd, 0 ));
	
				if ( GetFocus() == hWnd )
					focus_border( hWnd, TRUE );
	
				/*
				** clean up before exit
				*/
				
				EndPaint( hWnd, &ps );
				
				/*
				** Block all other painting
				*/
	
				return( 0L );
			}
		break;
		}

	default:

		break;

	}
	
	/*
	** Forward all unmodified messages to the old button
	** Button procedure.
	*/
	
	return(  CallWindowProc( OldButtonProc, hWnd, wMsg, wParam, lParam ));
}


/****************************************************************************
Function: WEP

Purpose:  Performs cleanup tasks when the DLL is unloaded.  WEP() is
	  called automatically by Windows when the DLL is unloaded (no
	  remaining tasks still have the DLL loaded).  It is strongly
	  recommended that a DLL have a WEP() function, even if it does
	  nothing but returns success (1), as in this example.

*******************************************************************************/
int FAR PASCAL WEP ( int bSystemExit )
{
	bSystemExit;

	return(1);
}

/*****************************************************************************
Function Name: LibMain
Purpose: This is the main entry point for the Image Window Library 
 
Entry: Standard Windows Stuff 

Returns: FALSE if unable to initialize

*****************************************************************************/
int FAR PASCAL LibMain( HANDLE hModule, WORD wDataSeg, 
 WORD cbHeapSize, LPSTR lpszCmdLine )
{
	wDataSeg;
	cbHeapSize;
	lpszCmdLine;

	/*
	** Save off module handle to use as instance later
	*/

	hInst = hModule;

	/*
	** Register a class for the enhanced button class
	*/

	if ( ! VCRButtonInit( hInst ) )
		return( FALSE );

	return( TRUE );
}
 